/*
File Name: BlogView.java
Purpose: Sets up the Blog View Activity, Contains function calls to the navigation drawer and displaying the website
Related Files Used: activity_blog_view.xml, content_blog_view.xml, app_bar_blog_view.xml
Activities Accessed: Bookmarks.java, MainActivity.java
Listed Functions: OnCreate, shouldOverrideUrlLoading, SavePage, onNavigationItemSelected
Other Notes: Originally, this displayed a fragment.  However, I was able to change it so that the blog was displayed
    within the content_blog_view file instead of the fragment.
 */

package com.example.codys.compsciproject;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

public class BlogView extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener{

    String url;         // Url of the project site
    String project;     // Name of the project
    int year;           // Year of the project
    String author;      // Author of the project
    WebView blogView;   // WebView that the blog will be displayed in

    /*
    Function Name: OnCreate
    Purpose: Initializes the activity.
    Function Type: Void
    Parameters Used: Bundle savedInstanceState (Refers to any data that was sent to this activity)
    Related Files: content_blog_view.xml, activity_blog_view.xml, app_bar_blog_view.xml, nav_bar.xml, bookmarks.txt
    Other Notes: SetJavaScript is suppressed to prevent the warning on enabling javascript in the webview from
        popping up.  The warning does not indicate a major problem, nor is it even a problem.
    */
    @SuppressLint("SetJavaScriptEnabled")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // All we need to do is call activity_blog_view, because it contains a call to content_blog_view and app_bar_blog_view as well
        setContentView(R.layout.activity_blog_view);

        // Grabs the data from Results / Bookmarks and loads it into the activity variables
        Bundle fromList = getIntent().getExtras();
        url = fromList.getString("LIST_URL");
        project = fromList.getString("LIST_PROJECT");
        year = fromList.getInt("LIST_YEAR");
        author = fromList.getString("LIST_NAME");

        // Captures the webview held in content_blog_view.xml
        blogView = (WebView)findViewById(R.id.webView);

        // Set the settings for webview
        if(blogView != null)
        {
            blogView.setWebViewClient(new CustomWebViewClient());
            WebSettings webSetting = blogView.getSettings();
            webSetting.setJavaScriptEnabled(true);  // Enable javascript on the websites
            webSetting.setBuiltInZoomControls(true);    // Allow zoom in/out methods
            webSetting.setUseWideViewPort(true);        // Shows the website as it would in the browser
            webSetting.setLoadWithOverviewMode(true);   // Load the site completely zoomed out
            blogView.loadUrl(url);
        }

        // Set up the toolbar at the top of the screen
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Grab the navigation drawer to apply open and close functions to it
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);

        // Sets up the navigation drawer to detect when the user taps on the hamburger (three bars in the top left corner)
        if(drawer != null)
            drawer.addDrawerListener(toggle);

        // Sets the item click events for the navigation drawer
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        if(navigationView != null)
            navigationView.setNavigationItemSelectedListener(this);

        // Allows the drawer to be opened and closed
        toggle.syncState();
    }

    /*
    Class Name: CustomWebViewClient
    Purpose: Contains the call to open the website
    Related Files: content_blog_view.xml, activity_blog_view.xml
    */
    private class CustomWebViewClient extends WebViewClient
    {
        /*
        Function Name: shouldOverrideUrlLoading
        Purpose: Loads the website within the web view component in content_blog_view.xml
        Function Type: Boolean
        Parameters Used: WebView view (Contains the ), String url (contains the url to the project site
        Related Files: content_blog_view.xml, activity_blog_view.xml
        Other Notes: The approach utilized for the blog view is based on the fragment approach.
            Because I was able to implement the blog view within content_blog_view, I was able to cut down
            on the amount of code and files used.
        */
        @Override
        public boolean shouldOverrideUrlLoading(WebView view, String url)
        {
            view.loadUrl(url);
            return true;
        }
    }

    /*
    Function Name: onBackPressed
    Purpose: If the navigation drawer is open, the back button will allow it to be closed.  Otherwise, if
        the user has navigated within the website they are viewing, the back button will take them to the
        previous page.
    Function Type: Void
    Parameters Used: None
    Related Files: activity_blog_view.xml, app_bar_blog_view.xml, nav_bar.xml, content_blog_view.xml
    Other Notes: None
    */
    @Override
    public void onBackPressed()
    {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        if(drawer != null && drawer.isDrawerOpen(GravityCompat.START))
            drawer.closeDrawer(GravityCompat.START);
        else if(blogView.canGoBack())
            blogView.goBack();
        else
            super.onBackPressed();
    }

    /*
    Function Name: SavePage
    Purpose: When the picfab icon (The yellow circle with the star) is tapped in the bottom right corner of the screen,
        the information with the blog is saved to the internal bookmarks text file.
    Function Type: Void
    Parameters Used: View x(Refers to content_main and the fields in it, and x is a placeholder for the second view word
        as to avoid confusion with this explanation)
    Related Files: bookmarks.txt, activity_blog_view.xml,
    Other Notes:
    */
    public void SavePage(View view)
    {
        try
        {
            String Result;  // Will be used to grab each line from the file
            String filepath = "MyFileStorage";  // Filepath to the internal database
            ContextWrapper contextWrapper = new ContextWrapper(getApplicationContext());  // Grabs the process the app is running in
            File directory = contextWrapper.getDir(filepath, Context.MODE_APPEND);  // Sets a file to the path to where the actual database is stored
            File file = new File(directory, "bookmarks.txt");   // Path to the actual database

            FileInputStream bm_file_in = new FileInputStream(file); // Stream to get information from the file
            InputStreamReader inputStreamReader = new InputStreamReader(bm_file_in);
            BufferedReader buff_read = new BufferedReader(inputStreamReader);
            StringBuilder buff_string = new StringBuilder();

            // While there is information in the file, read it into the result string
            while((Result = buff_read.readLine()) != null)
            {
                // Add the string to the buffer, then move on to the next
                buff_string.append(Result);
                buff_string.append("\r\n");
            }

            bm_file_in.close();

            String finalJson = buff_string.toString();  // Convert the buffer to a string
            String new_entry = "{" + project + ";" + url + ";" + author + ";" + Integer.toString(year) + "}" + "\r\n";
            String to_toast;

            // Check if the bookmark is already in the file.  Since the url is a unique value with each project, this is what
            //   the program searches for.
            if(finalJson.contains(url))
            {
                // If the file only had one entry in it, clear it.  Otherwise, replace the entry in the string with a blank
                to_toast = "Site has been removed from Bookmarks";
                if(finalJson.trim().length() == 0)
                    finalJson = "";
                else
                    finalJson = finalJson.replace(new_entry,"");
            }
            else
            {
                // If the bookmarks file was not empty, append the new entry onto what is there.  Otherwise, set the new entry
                //  as the finalJson string.
                to_toast = "Project successfully saved to Bookmarks";
                if(finalJson.trim().length() != 0)
                    finalJson = finalJson + new_entry;
                else
                    finalJson = new_entry;
            }

            if(file.delete())   // Clear the internal file
            {
                FileOutputStream bm_file_out = new FileOutputStream(file);
                bm_file_out.write(finalJson.getBytes());
                bm_file_out.close();
                // Used for the purpose of testing, ensuring that everything is saved to the bookmarks file in the right format
                /* Toast.makeText(getApplicationContext(), finalJson,
                    Toast.LENGTH_LONG).show(); */

                Toast.makeText(getApplicationContext(), to_toast,
                        Toast.LENGTH_LONG).show();
            }
            else
            {
                Toast.makeText(getApplicationContext(), "Error, internal database cleaning was unsuccessful",
                        Toast.LENGTH_LONG).show();
            }
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    /*
    Function Name: onNavigationItemSelected
    Purpose: Sets up Navigation Drawer item click events for the current activity
    Function Type: Boolean
    Parameters Used: MenuItem item (Refers to the navigation drawer that item that was selected)
    Related Files: nav_bar.xml, activity_blog_view.xml, app_bar_blog_view.xml
    Other Notes: The "Statement with empty body" warning does not cause any problems.  Thus, the
        warning is suppressed using the SuppressWarnings override at the beginning.
    */
    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item)
    {
        // Gets the id of the menu item selected
        int id = item.getItemId();

        if (id == R.id.nav_search)
        {
            // If "Home" was selected, call the activity
            Intent search_page = new Intent(this, MainActivity.class);
            startActivity(search_page);
        }
        else if (id == R.id.nav_bookmarks)
        {
            // If "Bookmarks" was selected, call the activity
            Intent bookmark_page = new Intent(this, Bookmarks.class);
            startActivity(bookmark_page);
        }
        else if (id == R.id.nav_about)
        {
            // If "About" was selected, display the toast containing the app information
            Toast.makeText(getApplicationContext(), R.string.about_string,
                    Toast.LENGTH_LONG).show();
        }

        // Grab the drawer itself
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        // As long as the drawer exists, close it
        if(drawer != null)
            drawer.closeDrawer(GravityCompat.START);

        return true;
    }

}
