/*
File Name: MainActivity.java
Purpose: Sets up the Search Page Activity, Contains function calls to the navigation drawer and to go to the Results Page
Related Files Used: activity_main.xml, content_main.xml, app_bar_main.xml
Activities Accessed: Bookmarks.java, Results.java
Listed Functions: ToResults, OnCreate, onBackPressed, onNavigationItemSelected
Other Notes: This is the Home Activity, as in this is the first activity that is loaded when the user boots up the app
 */

package com.example.codys.compsciproject;

import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

public class MainActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener
{

    /*
    Function Name: ToResults
    Purpose: When the "Go To Results" button is pressed, this function creates a bundle, loads all the values from the
        search fields into the bundle, and then calls the Results activity while sending the bundle on to Results
    Function Type: Void
    Parameters Used: View x(Refers to content_main and the fields in it, and x is a placeholder for the second view word
        as to avoid confusion with this explanation)
    Related Files: content_main.xml, Results.java
    Other Notes: A warning is raised with all of the uses of the EditText variable current in if statements.  These
        warnings have been handled with the catch loop towards the end.
    */
    public void ToResults(View view)
    {
        EditText current = null;
        Bundle ResBund = new Bundle();

        try
        {
            /* ResBund.putInt and ResBund.putString refer to taking the value held in the given field and adding it to the
                   ResBund bundle defined above.  After the try block, it will be packaged with the Results activity and
                   called with it.
             */

            // Grab the start year EditText field
            current = (EditText) findViewById(R.id.startyear);
            if (current.getText().toString().trim().length() == 0)
                ResBund.putInt("SEARCH_YEARSTART", 0);
            else
                ResBund.putInt("SEARCH_YEARSTART", Integer.parseInt(current.getText().toString()));

            // Grab the end year EditText field
            current = (EditText) findViewById(R.id.endyear);
            if (current.getText().toString().trim().length() == 0)
                ResBund.putInt("SEARCH_YEAREND", 9999);
            else
                ResBund.putInt("SEARCH_YEAREND", Integer.parseInt(current.getText().toString()));

            // Grab the author EditText field
            current = (EditText) findViewById(R.id.author);
            if (current.getText().toString().trim().length() == 0)
                ResBund.putString("SEARCH_STUDENT", " ");
            else
                ResBund.putString("SEARCH_STUDENT", current.getText().toString());

            // Grab the project name EditText field
            current = (EditText) findViewById(R.id.project_name);
            if (current.getText().toString().trim().length() == 0)
                ResBund.putString("SEARCH_TOPIC", " ");
            else
                ResBund.putString("SEARCH_TOPIC", current.getText().toString());

            // Grab the keywords EditText field
            current = (EditText) findViewById(R.id.keyword);
            if (current.getText().toString().trim().length() == 0)
                ResBund.putString("SEARCH_KEYWORD", " ");
            else
                ResBund.putString("SEARCH_KEYWORD", current.getText().toString());

        }
        catch(NullPointerException e)
        {
            e.printStackTrace();
        }

        Intent result_page = new Intent(this, Results.class);
        result_page.putExtras(ResBund); // Package the bundle with the Results activity
        startActivity(result_page); // Call the Results activity
    }

    /*
    Function Name: ClearFields
    Purpose: When the "Clear Fields" button has been pressed, set all of the EditText fields back to blank
    Function Type: Void
    Parameters Used: View x(Refers to content_main and the fields in it, and x is a placeholder for the second view word
        as to avoid confusion with this explanation)
    Related Files: content_main.xml
    Other Notes: A warning is raised with all of the uses of the EditText variable current in if statements.  These
        warnings have been handled with the catch loop towards the end.  In addition, this function is derived from
        the ToResults function
    */
    public void ClearFields(View view)
    {
        EditText current = null;

        try
        {
            // Grab the start year EditText field
            current = (EditText) findViewById(R.id.startyear);
            if (current.getText().toString().trim().length() != 0)
                current.setText("");

            // Grab the end year EditText field
            current = (EditText) findViewById(R.id.endyear);
            if (current.getText().toString().trim().length() != 0)
                current.setText("");

            // Grab the author EditText field
            current = (EditText) findViewById(R.id.author);
            if (current.getText().toString().trim().length() != 0)
                current.setText("");

            // Grab the project name EditText field
            current = (EditText) findViewById(R.id.project_name);
            if (current.getText().toString().trim().length() != 0)
                current.setText("");

            // Grab the keywords EditText field
            current = (EditText) findViewById(R.id.keyword);
            if (current.getText().toString().trim().length() != 0)
                current.setText("");

            // Let the user know all fields have been cleared
            Toast.makeText(getApplicationContext(), "All fields have been cleared",
                    Toast.LENGTH_LONG).show();
        }
        catch(NullPointerException e)
        {
            e.printStackTrace();
        }
    }

    /*
    Function Name: OnCreate
    Purpose: Initializes the activity.  Also, if the internal database does not exist, create it
    Function Type: Void
    Parameters Used: Bundle savedInstanceState (Refers to any data that was sent to this activity)
    Related Files: content_main.xml, activity_main.xml, app_bar_main.xml, nav_bar.xml, bookmarks.txt
    Other Notes:
    */
    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        // All we need to do is call activity_main, because it contains a call to content_main and app_bar_main as well
        setContentView(R.layout.activity_main);

        // Set up the toolbar at the top of the screen
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Grab the navigation drawer to apply open and close functions to it
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);

        // Sets up the navigation drawer to detect when the user taps on the hamburger (three bars in the top left corner)
        if(drawer != null)
            drawer.addDrawerListener(toggle);

        // Allows the drawer to be opened and closed
        toggle.syncState();

        // Sets the item click events for the navigation drawer
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        if(navigationView != null)
            navigationView.setNavigationItemSelectedListener(this);

        // This is the check to confirm a file exists
        String filepath = "MyFileStorage";
        ContextWrapper contextWrapper = new ContextWrapper(getApplicationContext());
        File directory = contextWrapper.getDir(filepath, Context.MODE_PRIVATE);
        File file = new File(directory, "bookmarks.txt");
        //file.delete();  // Used during testing to clear the bookmarks file.

        if(!file.exists())
        {
            // If the internal bookmarks file does not exist, create it
            try
            {
                FileOutputStream bm_file_out = new FileOutputStream(file);
                bm_file_out.close();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    /*
    Function Name: onBackPressed
    Purpose: If the navigation drawer is open, the back button will allow it to be closed
    Function Type: Void
    Parameters Used: None
    Related Files: activity_main.xml, app_bar_main.xml, nav_bar.xml
    Other Notes: None
    */
    @Override
    public void onBackPressed()
    {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if(drawer != null) {
            if (drawer.isDrawerOpen(GravityCompat.START)) {
                drawer.closeDrawer(GravityCompat.START);
            } else {
                super.onBackPressed();
            }
        }
    }

    /*
    Function Name: onNavigationItemSelected
    Purpose: Sets up Navigation Drawer item click events for the current activity
    Function Type: Boolean
    Parameters Used: MenuItem item (Refers to the navigation drawer that item that was selected)
    Related Files: nav_bar.xml, activity_main.xml, app_bar_main.xml
    Other Notes: The "Statement with empty body" warning does not cause any problems.  Thus, the
        warning is suppressed using the SuppressWarnings override at the beginning.
    */
    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item)
    {
        // Gets the id of the menu item selected
        int id = item.getItemId();

        if (id == R.id.nav_search)
        {
            // If "Home" was selected, call the activity
            Intent search_page = new Intent(this, MainActivity.class);
            startActivity(search_page);
        }
        else if (id == R.id.nav_bookmarks)
        {
            // If "Bookmarks" was selected, call the activity
            Intent bookmark_page = new Intent(this, Bookmarks.class);
            startActivity(bookmark_page);
        }
        else if (id == R.id.nav_about)
        {
            // If "About" was selected, display the toast containing the app information
            Toast.makeText(getApplicationContext(), R.string.about_string,
                    Toast.LENGTH_LONG).show();
        }

        // Grab the drawer itself
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        // As long as the drawer exists, close it
        if(drawer != null)
            drawer.closeDrawer(GravityCompat.START);

        return true;
    }
}
